@@ -1,5 +1,5 @@
-/* Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993
-	Free Software Foundation, Inc.
+/* Target file hash table management for GNU Make.
+Copyright (C) 1988, 89, 90, 91, 92, 93, 94, 1995 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "dep.h"
 #include "file.h"
 #include "variable.h"
+#include <assert.h>
 
 
 /* Hash table of files the makefile knows how to make.  */
@@ -114,6 +115,7 @@ enter_file (name)
   else
     {
       /* There is already a double-colon entry for this file.  */
+      new->double_colon = f;
       while (f->prev != 0)
 	f = f->prev;
       f->prev = new;
@@ -188,7 +190,8 @@ file_hash_enter (file, name, oldhash, oldname)
 
   /* Give FILE its new name.  */
 
-  for (f = file; f != 0; f = f->prev)
+  file->name = name;
+  for (f = file->double_colon; f != 0; f = f->prev)
     f->name = name;
 
   if (oldfile == 0)
@@ -292,7 +295,9 @@ remove_intermediates (sig)
   register struct file *f;
   char doneany;
   
-  if (!sig && just_print_flag)
+  if (question_flag || touch_flag)
+    return;
+  if (sig && just_print_flag)
     return;
 
   doneany = 0;
@@ -301,7 +306,11 @@ remove_intermediates (sig)
       if (f->intermediate && (f->dontcare || !f->precious))
 	{
 	  int status;
-	  if (just_print_flag)
+	  if (f->update_status == -1)
+	    /* If nothing would have created this file yet,
+	       don't print an "rm" command for it.  */
+	    continue;
+	  else if (just_print_flag)
 	    status = 0;
 	  else
 	    {
@@ -381,6 +390,47 @@ snap_deps ()
   f = lookup_file (".EXPORT_ALL_VARIABLES");
   if (f != 0 && f->is_target)
     export_all_variables = 1;
+
+  f = lookup_file (".IGNORE");
+  if (f != 0 && f->is_target)
+    {
+      if (f->deps == 0)
+	ignore_errors_flag = 1;
+      else
+	for (d = f->deps; d != 0; d = d->next)
+	  for (f2 = d->file; f2 != 0; f2 = f2->prev)
+	    f2->command_flags |= COMMANDS_NOERROR;
+    }
+
+  f = lookup_file (".SILENT");
+  if (f != 0 && f->is_target)
+    {
+      if (f->deps == 0)
+	silent_flag = 1;
+      else
+	for (d = f->deps; d != 0; d = d->next)
+	  for (f2 = d->file; f2 != 0; f2 = f2->prev)
+	    f2->command_flags |= COMMANDS_SILENT;
+    }
+
+  f = lookup_file (".POSIX");
+  if (f != 0 && f->is_target)
+    posix_pedantic = 1;
+}
+
+/* Set the `command_state' member of FILE and all its `also_make's.  */
+
+void
+set_command_state (file, state)
+     struct file *file;
+     int state;
+{
+  struct dep *d;
+
+  file->command_state = state;
+
+  for (d = file->also_make; d != 0; d = d->next)
+    d->file->command_state = state;
 }
 
 /* Print the data base of files.  */
@@ -448,6 +498,10 @@ print_file (f)
 	  puts ("#  Successfully updated.");
 	  break;
 	case 1:
+	  assert (question_flag);
+	  puts ("#  Needs to be updated (-q is set).");
+	  break;
+	case 2:
 	  puts ("#  Failed to be updated.");
 	  break;
 	default:
