@@ -1,5 +1,5 @@
 /* Get the system load averages.
-   Copyright (C) 1985, 86, 87, 88, 89, 91, 92, 93, 1994
+   Copyright (C) 1985, 86, 87, 88, 89, 91, 92, 93, 1994, 1995
    	Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
@@ -59,6 +59,11 @@
    We also #define LDAV_PRIVILEGED if a program will require
    special installation to be able to call getloadavg.  */
 
+/* This should always be first.  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <sys/types.h>
 
 /* Both the Emacs and non-Emacs sections want this.  Some
@@ -69,10 +74,6 @@
 #endif
 
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 /* Exclude all the code except the test program at the end
    if the system has its own `getloadavg' function.
 
@@ -209,6 +210,10 @@ extern int errno;
 #define LOAD_AVE_TYPE long
 #endif
 
+#ifdef _AIX
+#define LOAD_AVE_TYPE long
+#endif
+
 #endif /* No LOAD_AVE_TYPE.  */
 
 #ifdef OSF_ALPHA
@@ -253,6 +258,10 @@ extern int errno;
 #define FSCALE 100.0
 #endif
 
+#ifdef _AIX
+#define FSCALE 65536.0
+#endif
+
 #endif	/* Not FSCALE.  */
 
 #if !defined (LDAV_CVT) && defined (FSCALE)
@@ -314,6 +323,10 @@ extern int errno;
 #define NLIST_STRUCT
 #endif
 
+#ifdef _AIX
+#define NLIST_STRUCT
+#endif
+
 #endif /* defined (NLIST_STRUCT) */
 
 
@@ -339,7 +352,7 @@ extern int errno;
 #define LDAV_SYMBOL "_Loadavg"
 #endif
 
-#if !defined(LDAV_SYMBOL) && ((defined(hpux) && !defined(hp9000s300)) || defined(_SEQUENT_) || defined(SVR4) || defined(ISC) || defined(sgi) || (defined (ardent) && defined (titan)))
+#if !defined(LDAV_SYMBOL) && ((defined(hpux) && !defined(hp9000s300)) || defined(_SEQUENT_) || defined(SVR4) || defined(ISC) || defined(sgi) || (defined (ardent) && defined (titan)) || defined (_AIX))
 #define LDAV_SYMBOL "avenrun"
 #endif
 
@@ -789,23 +802,29 @@ getloadavg (loadavg, nelem)
 #endif /* NLIST_STRUCT */
 
 #ifndef SUNOS_5
-      if (nlist (KERNEL_FILE, nl) >= 0)
-	/* Omit "&& nl[0].n_type != 0 " -- it breaks on Sun386i.  */
-	{
-#ifdef FIXUP_KERNEL_SYMBOL_ADDR
-	  FIXUP_KERNEL_SYMBOL_ADDR (nl);
+      if (
+#ifndef _AIX
+	  nlist (KERNEL_FILE, nl)
+#else  /* _AIX */
+	  knlist (nl, 1, sizeof (nl[0]))
 #endif
-	  offset = nl[0].n_value;
-	}
-#endif  /* !SUNOS_5 */
-#else /* sgi */
-      int ldav_off;
+	  >= 0)
+	  /* Omit "&& nl[0].n_type != 0 " -- it breaks on Sun386i.  */
+	  {
+#ifdef FIXUP_KERNEL_SYMBOL_ADDR
+	    FIXUP_KERNEL_SYMBOL_ADDR (nl);
+#endif
+	    offset = nl[0].n_value;
+	  }
+#endif /* !SUNOS_5 */
+#else  /* sgi */
+	  int ldav_off;
 
-      ldav_off = sysmp (MP_KERNADDR, MPKA_AVENRUN);
-      if (ldav_off != -1)
-	offset = (long) ldav_off & 0x7fffffff;
+	  ldav_off = sysmp (MP_KERNADDR, MPKA_AVENRUN);
+	  if (ldav_off != -1)
+	  offset = (long) ldav_off & 0x7fffffff;
 #endif /* sgi */
-    }
+	}
 
   /* Make sure we have /dev/kmem open.  */
   if (!getloadavg_initialized)
