@@ -1,5 +1,5 @@
 /* Reading and parsing of makefiles for GNU Make.
-Copyright (C) 1988, 89, 90, 91, 92, 93, 94 Free Software Foundation, Inc.
+Copyright (C) 1988, 89, 90, 91, 92, 93, 94, 1995 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -631,7 +631,7 @@ read_makefile (filename, flags)
 	;
       else if (lb.buffer[0] == '\t')
 	{
-	  p = lb.buffer;
+	  p = collapsed;	/* Ignore comments.  */
 	  while (isblank (*p))
 	    ++p;
 	  if (*p == '\0')
@@ -723,6 +723,11 @@ read_makefile (filename, flags)
 	      else
 		break;
 	    }
+#ifdef __MSDOS__
+	  /* For MS-DOS, skip a "C:\...".  */
+	  if (p != 0 && p[1] == '\\' && isalpha (p[-1]))
+	    p = 0;
+#endif
 	  if (p != 0)
 	    {
 	      struct nameseq *target;
@@ -1432,9 +1437,9 @@ record_files (filenames, pattern, pattern_percent, deps, commands_started,
    one, or nil if there are none.  */
 
 char *
-find_char_unquote (string, stopchar, blank)
+find_char_unquote (string, stopchars, blank)
      char *string;
-     int stopchar;
+     char *stopchars;
      int blank;
 {
   unsigned int string_len = strlen (string);
@@ -1442,19 +1447,12 @@ find_char_unquote (string, stopchar, blank)
 
   while (1)
     {
-      if (blank)
-	{
-	  while (*p != '\0' && *p != stopchar && !isblank (*p))
-	    ++p;
-	  if (*p == '\0')
-	    break;
-	}
-      else
-	{
-	  p = index (p, stopchar);
-	  if (p == 0)
-	    break;
-	}
+      while (*p != '\0' && index (stopchars, *p) == 0
+	     && (!blank || !isblank (*p)))
+	++p;
+      if (*p == '\0')
+	break;
+
       if (p > string && p[-1] == '\\')
 	{
 	  /* Search for more backslashes.  */
@@ -1488,7 +1486,7 @@ char *
 find_percent (pattern)
      char *pattern;
 {
-  return find_char_unquote (pattern, '%', 0);
+  return find_char_unquote (pattern, "%", 0);
 }
 
 /* Search STRING for an unquoted ; that is not after an unquoted #.  */
@@ -1497,8 +1495,11 @@ static char *
 find_semicolon (string)
      char *string;
 {
-  return (find_char_unquote (string, '#', 0) == 0
-	  ? find_char_unquote (string, ';', 0) : 0);
+  char *match = find_char_unquote (string, ";#", 0);
+  if (match != 0 && *match == '#')
+    /* We found a comment before a semicolon.  No match.  */
+    match = 0;
+  return match;
 }
 
 /* Parse a string into a sequence of filenames represented as a
@@ -1526,6 +1527,9 @@ parse_file_seq (stringp, stopchar, size, strip)
   register char *p = *stringp;
   char *q;
   char *name;
+  char stopchars[2];
+  stopchars[0] = stopchar;
+  stopchars[1] = '\0';
 
   while (1)
     {
@@ -1537,7 +1541,12 @@ parse_file_seq (stringp, stopchar, size, strip)
 	break;
       /* Yes, find end of next name.  */
       q = p;
-      p = find_char_unquote (q, stopchar, 1);
+      p = find_char_unquote (q, stopchars, 1);
+#ifdef __MSDOS__
+      /* For MS-DOS, skip a "C:\...".  */
+      if (stopchar == ':' && p != 0 && p[1] == '\\' && isalpha (p[-1]))
+	p = 0;
+#endif
       if (p == 0)
 	p = q + strlen (q);
 
@@ -1573,7 +1582,9 @@ parse_file_seq (stringp, stopchar, size, strip)
      an elt further down the chain (i.e., previous in the file list)
      with an unmatched `(' (e.g., "lib(mem").  */
 
-  for (new1 = new, lastnew1 = 0; new1 != 0; lastnew1 = new1, new1 = new1->next)
+  new1 = new;
+  lastnew1 = 0;
+  while (new1 != 0)
     if (new1->name[0] != '('	/* Don't catch "(%)" and suchlike.  */
 	&& new1->name[strlen (new1->name) - 1] == ')'
 	&& index (new1->name, '(') == 0)
@@ -1654,14 +1665,21 @@ parse_file_seq (stringp, stopchar, size, strip)
 		name = concat (libname, new1->name, ")");
 		free (new1->name);
 		new1->name = name;
+		lastnew1 = new1;
 		new1 = new1->next;
 	      }
-
-	    if (new1 == 0)
-	      /* We might have slurped up the whole list,
-		 and continuing the loop would dereference NEW1.  */
-	      break;
 	  }
+	else
+	  {
+	    /* No frobnication happening.  Just step down the list.  */
+	    lastnew1 = new1;
+	    new1 = new1->next;
+	  }
+      }
+    else
+      {
+	lastnew1 = new1;
+	new1 = new1->next;
       }
 
 #endif
