@@ -15,14 +15,22 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>		/* Some systems need this for <signal.h>.  */
 #include <signal.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
+/* Some systems declare `sys_siglist in <unistd.h>; if
+   configure defined SYS_SIGLIST_DECLARED, it may expect
+   to find the declaration there.  */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
 #endif
 
+
 /* Some systems do not define NSIG in <signal.h>.  */
 #ifndef	NSIG
 #ifdef	_NSIG
